﻿using LDBproject.Repositories;
using Unity;

namespace LDBproject.AdditionalForms;

public partial class BookListF : Form
{
    private readonly IUnityContainer _container;
    private readonly IBookRep _bookR;

    public BookListF(IUnityContainer container, IBookRep bookR)
    {
        InitializeComponent();
        _bookR = bookR ?? throw new ArgumentNullException(nameof(bookR));
        _container = container ?? throw new ArgumentNullException(nameof(container));
    }

    private void BookListF_Load(object sender, EventArgs e)
    {
        try
        {
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Book List Form [ Error while saving ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void AddBtn_Click(object sender, EventArgs e)
    {
        try
        {
            _container.Resolve<BookF>().ShowDialog();
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Book List Form [ Error while adding element ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void UpdBtn_Click(object sender, EventArgs e)
    {
        if (!GetIDFromRow(out var findID))
        {
            return;
        }

        try
        {
            var form = _container.Resolve<BookF>();
            form.ID = findID;
            form.ShowDialog();
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Book List Form [ Error while updating element ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ReloadList() => DataGV.DataSource = _bookR.GetBookList();

    private void DelBtn_Click(object sender, EventArgs e)
    {
        if (!GetIDFromRow(out var foundID))
        {
            return;
        }

        if (MessageBox.Show("Remove element?", "Deleting", MessageBoxButtons.YesNo) != DialogResult.Yes)
        {
            return;
        }

        try
        {
            _bookR.DeleteBook(foundID);
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Book List Form [ Error while removing element ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private bool GetIDFromRow(out int id)
    {
        id = 0;
        if (DataGV.SelectedRows.Count < 1)
        {
            MessageBox.Show("[ Error : Book doesn't exist ]", "<ERROR>",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        id = Convert.ToInt32(DataGV.SelectedRows[0].Cells["BookID"].Value);
        return true;
    }
}
