﻿using LDBproject.Repositories;
using Unity;

namespace LDBproject.AdditionalForms;

public partial class CustomerListF : Form
{
    private readonly IUnityContainer _container;
    private readonly ICustomerCardsRep _custR;

    public CustomerListF(IUnityContainer container, ICustomerCardsRep customeR)
    {
        InitializeComponent();
        _custR = customeR ?? throw new ArgumentNullException(nameof(customeR));
        _container = container ?? throw new ArgumentNullException(nameof(container));
    }

    private void CustomerListF_Load(object sender, EventArgs e)
    {
        try
        {
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Customer List Form [ Error while saving ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void AddBtn_Click(object sender, EventArgs e)
    {
        try
        {
            _container.Resolve<CustomerF>().ShowDialog();
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Customer List Form [ Error while adding element ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void DelBtn_Click(object sender, EventArgs e)
    {
        if (!GetIDFromRow(out var foundID))
        {
            return;
        }

        if (MessageBox.Show("Remove element?", "Deleting", MessageBoxButtons.YesNo) != DialogResult.Yes)
        {
            return;
        }

        try
        {
            _custR.DeleteCard(foundID);
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Customer List Form [ Error while removing element ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ReloadList() =>
        DataGV.DataSource = _custR.GetCards();

    private bool GetIDFromRow(out int id)
    {
        id = 0;
        if (DataGV.SelectedRows.Count < 1)
        {
            MessageBox.Show("[ Error : Customer doesn't exist ]", "<ERROR>",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        id = Convert.ToInt32(DataGV.SelectedRows[0].Cells["CardID"].Value);
        return true;
    }

    private void UpdBtn_Click(object sender, EventArgs e)
    {
        if (!GetIDFromRow(out var findID))
        {
            return;
        }

        try
        {
            var form = _container.Resolve<CustomerF>();
            form.ID = findID;
            form.ShowDialog();
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Customer List Form [ Error while updating element ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }
}

