﻿using LDBproject.Repositories;
using Unity;

namespace LDBproject.AdditionalForms;

public partial class EmployeesF : Form
{
    private readonly IUnityContainer _container;
    private readonly ILibrarianRep _libR;

    public EmployeesF(IUnityContainer container, ILibrarianRep libR)
    {
        InitializeComponent();
        _libR = libR ?? throw new ArgumentNullException(nameof(libR));
        _container = container ?? throw new ArgumentNullException(nameof(container));
    }

    private void EmployeesF_Load(object sender, EventArgs e)
    {
        try
        {
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "[ Error while saving ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void AddBtn_Click(object sender, EventArgs e)
    {
        try
        {
            _container.Resolve<EmployeeF>().ShowDialog();
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "[ Error while adding element ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void UpdBtn_Click(object sender, EventArgs e)
    {
        if (!GetiDFromRow(out var findID))
        {
            return;
        }

        try
        {
            var form = _container.Resolve<EmployeeF>();
            form.ID = findID;
            form.ShowDialog();
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "[ Error while updating element ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void DelBtn_Click(object sender, EventArgs e)
    {
        if (!GetiDFromRow(out var foundID))
        {
            return;
        }

        if (MessageBox.Show("Remove element?", "Deleting", MessageBoxButtons.YesNo) != DialogResult.Yes)
        {
            return;
        }

        try
        {
            _libR.DeleteCard(foundID);
            ReloadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "[ Error while removing element ]", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ReloadList() => DataGV.DataSource = _libR.GetCards();

    private bool GetiDFromRow(out int id)
    {
        id = 0;
        if (DataGV.SelectedRows.Count < 1)
        {
            MessageBox.Show("[ Error : element doesn't exist ]", "<ERROR>",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        id = Convert.ToInt32(DataGV.SelectedRows[0].Cells["CardID"].Value);
        return true;
    }
}

